from .features import FEATURE_PARAMS, FEATURE_SET

# Model params
ALGORITHM_NAME = 'algorithm_name'
PIPELINE_STEP = 'pipeline_step'
## Config params
NDIM = 'ndim'

# Analyser & Plotter params
PLOT_TYPE = 'plot_type'
PLOT_TICS = 'plot_tics'
INTERACTIVE = 'interactive'
PROJECTION = 'projection'
EXPLAINED_VAR = 'explained_variance'
## Plot Types params
HEAT_MAP = 'heat_map'
COLOR_MAP = 'color_map'
PLOT_3D_MAP = '3d_map'
EXPL_VAR_PLOT = 'explained_variance_plot'

# Fitting params
N_COMPONENTS = 'n_components'

# File system and params
FILENAME = 'filename'
FOLDER_PATH = 'folder_path'
PARAMS = 'params'
INPUT_PATH = 'input_path'
OUTPUT_PATH = 'output_path'
