def column_mappings():

    column_names_short = {
    'rutpt': 'ratio_unique_traces_per_trace',
    'rmcv': 'ratio_most_common_variant',
    'tlcv': 'trace_len_coefficient_variation',
    'mvo': 'mean_variant_occurrence',
    'enve': 'epa_normalized_variant_entropy',
    'ense': 'epa_normalized_sequence_entropy',
    'eself': 'epa_sequence_entropy_linear_forgetting',
    'enself': 'epa_normalized_sequence_entropy_linear_forgetting',
    'eseef': 'epa_sequence_entropy_exponential_forgetting',
    'enseef': 'epa_normalized_sequence_entropy_exponential_forgetting'
    }

    return column_names_short